import os, glob
import pytest
import subprocess
import pytest
from NFSyndication import __main__ as NFS_init
from NFSyndication import parser as PS
# test@
# Change the action associated with your option to action='store'
"""
def test_conf():
    #We use these conditions to check the statement
    subscriptions = [
     'http://feedpress.me/512pixels',
     'http://www.leancrew.com/all-this/feed/',
     'http://ihnatko.com/feed/',
     'http://blog.ashleynh.me/feed']
  
    with open(f'feeds.txt', 'w', encoding='utf8') as f:
     f.write(",".join(subscriptions).replace(',', '\n'))
    return NFS_init.run()
    
def test_entrypoint():
    #Then initialize code
    return test_conf()
    
def pytest_configure():
    pytest.something = test_entrypoint()
"""

@pytest.fixture
def test_client():
    configure_app(flask_app, config_name=Environments.TESTS)
    # I use this a lot for custom Test client classes to inject custom things
    my_project.app.test_client_class = CustomApiTestClient
    client = ''
    yield client


#!/usr/bin/python 
import sys
import logging
import colorful as cf
from jinja2 import Template, Environment, FileSystemLoader
from jinja2.exceptions import *
from feedparser import parse as parseURL
import json
import os
import shutil

cfail = cf.bold_red

def init():
  logging.info("Fetching...")
  try:
    from . import nfs_main
  except Exception as e:
    exception = cfail(
                  "Error while parsing: {exceptname}: {errmessage}".format(exceptname=str(e.__class__.__name__), errmessage=e)
                   )
    print(exception)
    sys.exit(1)
  except KeyboardInterrupt:
    print('Session terminated. Operation aborted by the user.')
    
class GetFeedDataPerConfiguration(object):
    def __init__(self, eachUrl, tempPath, emdF, fPath, outputPath, templFileLocation, outPath, pageTitle, outJSON=False, jsonFile=None):
        """ Request and parse all of the feeds, saving them in self.feeds"""
        self.outJSON = outJSON
        self.feeds = []
        self.outputP = os.path.join(emdF,os.path.sep).replace(os.path.sep, '{}/'.format(emdF))
        #self.dir_path = os.path.dirname(os.path.realpath(__file__))
        self.env = Environment(loader=FileSystemLoader(tempPath))
        self.fetch_feeds(eachUrl)
        self.empty_public(emdF)
        self.copy_static(fPath, outputPath)
        self.render_page(templFileLocation, outPath, pageTitle)
        if bool(outJSON) == True and type(jsonFile)==str:
         """ Ensure that the file would be written in JSON format"""
         self.outputJson(jsonFile, self.feeds)
         print('Renering JSON')
        elif bool(outJSON) == True and not jsonFile:
         print(cfail("Missing or undefined argument: jsonFile"))
         sys.exit(1)
        elif bool(outJSON) == True and not type(jsonFile)==str:
         print(
            cfail('Error: must be string type, not {}'.format(type(jsonFile)))
            )
         sys.exit(1)
        
    def fetch_feeds(self, URLs):
        """ Request and parse all of the feeds, saving them in self.feeds """
        from .extras import fetch_content
        for url in URLs:
          try:
            print(f"Fetching {url}")
            self.feedURL = parseURL(url)
            self.feeds.append(self.feedURL)
            fetch_content(url)
          except Exception as er:
            print('Error while looking for URL feed: {}'.format(er) )
            
    def empty_public(self, sf):
        """ Ensure the public directory is empty before generating. """
        try:
          shutil.rmtree(sf)          
          os.mkdir(sf)
        except:
            print(cfail(f"Error cleaning up old files."))

    def copy_static(self, fPath, outputPath):
        """ Copy static assets to the public directory """
        try:
            shutil.copytree('{}'.format(fPath), '{}'.format(os.path.join(self.outputP,outputPath)))
        except:
            print(cfail(f"Error copying static files."))

    def render_page(self, templFileLocation, outputPath, pageTitle):
       print("Rendering page to static file.")
       try:
          template = self.env.get_template(templFileLocation)
       except (TemplateSyntaxError, TemplateNotFound, UndefinedError, TemplateAssertionError) as error:
          print(cfail("Error while reading template: {}".format(str(error))))
          sys.exit(1)
       with open('{}'.format(os.path.join(self.outputP,outputPath)), 'w+', encoding='utf8') as file:
           try:
            html = template.render(
                title = '{}'.format(pageTitle),
                feeds = self.feeds
            )
            file.write(html)
           except UnboundLocalError:
             print(cfail('Error while writing output file: {}'.format(outputPath)))
             sys.exit(1)
            
    def outputJson(self, trh, jsonInput):
         try:
           with open(os.path.join(self.outputP,'{}.json'.format(trh)), 'w', encoding='utf8') as jsonF:
             self.jsonFile = jsonF
             json.dump(jsonInput, jsonF, ensure_ascii=False, indent=4)
         except TypeError:
             print('Error: {} at {hrefURL}'.format(self.feedURL.bozo_exception,hrefURL=self.feedURL.href)) 
             del self.feedURL             
           


# encoding = utf-8
"""
This file contains the logic for filtering/munging posts.  It's kept in
a separate file from the main feed parsing logic so the commit history
for nfs_main.py doesn't get polluted with nitpicks and tweaks.

"""
import collections
from datetime import datetime, timedelta
#from time import mktime
#def format_datetime(struct_time):
    #return datetime.fromtimestamp(mktime(struct_time))
import colorful as cf
import logging
import os
import json
import pytz
import sys
import time
from typing import NamedTuple
from configparser import ConfigParser

from colorama import init, Fore, Back, Style
from . import parser

argcomp = parser.parse_args()

init(convert=True)

config = ConfigParser()

# Date and time setup. I want only posts from "today" and "yesterday",
# where the day lasts until 2 AM.
TIMEZONE = config.get(section='default', option='timezone', fallback='GMT')

# Get the current time in the home timezone, then step back to include
# the last two days.
home_tz = pytz.timezone(TIMEZONE)
dt = datetime.now(home_tz)
if dt.hour < 2:
    dt -= timedelta(hours=72)
else:
    dt -= timedelta(hours=48)
start = dt.replace(hour=0, minute=0, second=0, microsecond=0)

# Convert this time back into UTC.
utc = pytz.utc
START = start.astimezone(utc) 

# List of keywords to filter
FILTER_WORDS = ['*']

cfail = cf.bold_red
cfhighlight = cf.bold_blue

def fetch_content(url):
   import feedparser
   feedJSON = []
   try:
    feed = feedparser.parse(url)
    feedJSON.append(feed)
    print("\nFeed title:", feed.feed.title or None)
    print("Link:", feed.feed.link or None)

    if hasattr(feed.feed, 'subtitle'):
        print("Subtitle:", feed.feed.subtitle)

    #print("Updated:", feed.feed.updated)
    #print("Updated (parsed):", format_datetime(feed.feed.updated_parsed))
    #print("Feed ID:", rss_url.feed.id)
    print('-----------------------------------------------')
    print("\nEntries:")
   
    for entry in feed.entries:
       print(cfhighlight(f" * Title: {entry.title}"))
       print(cfhighlight(f"   Link: {entry.link}"))
       print(cfhighlight(f"   Published: {entry.published}"))
       print(cfhighlight(f"   Updated: {entry.updated}"))
       print(cfhighlight(f"   Summary length: {len(entry.summary) or None} \n"))
       #print("   Content items count:", len(entry.content))
          
   except Exception as e:
    print(Fore.RED + 'Error occured while parsing some URLs: {feedurl} '.format(feedurl=url) + 'HTTP {return_code}'.format(return_code=feed.status))
    print('{}: {}'.format(str(e.__class__.__name__),str(e)))
    print(Style.RESET_ALL)
   except KeyboardInterrupt:
    print('Operation aborted.')
    sys.exit()
   else:
    print(cf.bold_green("   Respone: {}\n".format(feed.status)))
   finally:
    try:
     feedHeaders = json.dumps(feed.headers, indent=4, sort_keys=True)
     logging.info('Output headers JSON: \n' + feedHeaders)
    except Exception as e:
      print(e.__class__.__name__)

templateContent = """
<html>
<meta charset="UTF-8" />
<meta name="viewport" content="width=device-width" />
<head>
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <meta name="apple-mobile-web-app-capable" content="yes" />
<style type="text/css">
 {{ cssText }}
</style>
<title>News Topics: Today’s RSS</title>
<body>
  <aside>
    <div id="header_text"><h1><a href="javascript:window.location.reload()">News Stories</a></h1></div>
  </aside>
  <ul class="rss">
  {% for post in posts|sort|reverse %}
  <li>
    <div class="article_title">
      <h3 class="{% if post.permalink %}link{% else %}full{% endif %}post_title"><a href="{{ post.link }}">{{ post.title }}</a>{% if post.permalink %} <span class="linkpost_arrow">→</span>{% endif %}</h3>
    </div>
    <div class="article_meta"><p>
      On {{ post.time.strftime('%d %B %Y').strip('') }} at {{ post.time.strftime('%I:%M&thinsp;%p').strip('').lower() }}
      • {{ post.blog }}
      {% if post.author and post.blog != post.author %}
        • by {{ post.author }}
      {% endif %}
      {% if post.permalink %}
       • <span class="permalink"><a href="{{ post.permalink }}">∞</a></span>
      {% endif %}
    </p></div>
    {{ post.body|safe }}

    {# Put a line between posts, but only if this isn't the last one #}
    {% if loop.index != posts|count %}<hr class="between_posts"/>{% endif %}
  </li>
  {% endfor %}
  </ul>
  <footer>
    <p>
        Last updated on {{ time.strftime('%d %B %Y').strip('') }} at {{ time.strftime('%I:%M&thinsp;%p').strip('').lower() }}.
        Made with Python app & Jinja2 (based on script by configuration of the project)
    </p>
  </footer>
</body>
</html>
"""

class Post(NamedTuple):
  time: str
  blog: str
  title: str
  author: str
  link: str
  body: str

#ExtendedPost = NamedTuple('Post', [('time', str), ('blog', str), ('title',str), ('author',str), ('link', str), ('body',str), ('permalink', str)])

class ExtendedPost(NamedTuple):
  time: str
  blog: str
  title: str
  author: str
  link: str
  body: str
  permalink:str

def check_template():
  path_dest = os.path.realpath(__file__)
  parent_directory = os.path.dirname(path_dest)
  for root, dirs, files in os.walk(parent_directory):
    for file in files:
        if file == 'template.html':
            template_root = os.path.join(root, file)
            print("Template file {}?: {} {}".format(root, file, os.path.isfile(template_root)))
        else:
            print("Template file?: {} result is {boolean}".format(root, boolean=False))

def process_entry(entry, blog):
    """
    Coerces an entry from feedparser into a Post tuple.
    Returns None if the entry should be excluded.
    """
    # Get the date of the post.  If it was published more than two days
    # ago, drop the entry. Now, the feed entries is being analyzed and processed before reading in SUBSCRIPTIONS.
    try:
        when = entry['updated_parsed']
    except KeyError:
        when = entry['published_parsed']
    when = utc.localize(datetime.fromtimestamp(time.mktime(when)))
    
    if argcomp.comparator_filter:
      if when < START:
        return  

    title = entry['title']
    
    try:
        author = entry['author']
    except KeyError:
        author = ', '.join(a['name'] for a in entry.get('authors', []))
    link = entry['link']
    try:
        body = entry['content'][0]['value']
    except KeyError:
        body = entry['summary']
            
    postField = Post(when, blog, title, author, link, body)
    return normalise_post(postField)
    
def normalise_post(post):
    """
    This function takes a post and a blog, and applies some
    transformations to normalise the text.  This is mostly based on
    special cases and lots of if statements.
    It returns an ExtendedPost tuple, which includes fields not found
    in the regular Post.
    It may also return None, which means this post should be hidden.
    """
    blog = post.blog

    if any(word.lower() in post.body.lower() for word in FILTER_WORDS):
        return None
    
    """ 
       --------------------- NO LONGER NEEDED ---------------------
       
    if (blog == 'Marco.org'):
        if ('coffee' in post.body):
            return None
        if post.title.startswith(u'→'):
            title = post.title[2:]
            body = remove_final_link(post.body)
            permalink = extract_last_link(post.body)
            return ExtendedPost(post.time, post.blog, title, post.author,
                                post.link, body, permalink)

    elif (blog == 'Daring Fireball') and u'★' in post.body:
        body = remove_final_link(post.body)
        permalink = extract_last_link(post.body)
        return ExtendedPost(post.time, post.blog, post.title, post.author,
                            post.link, body, permalink)

    elif (blog == 'Erica Sadun') and (post.author == 'erica'):
        return ExtendedPost(post.time, post.blog, post.title,
                            None, post.link, post.body, None)
                            
       --------------------- NO LONGER NEEDED ---------------------
    """

    return ExtendedPost(*post, permalink=None)
   

def remove_final_link(html_text):
    return html_text.rsplit('<a', maxsplit=1)[0]


def extract_last_link(html_text):
    return html_text.rsplit('"', maxsplit=2)[-2]


import os, shutil
import feedparser
from .extras import fetch_content
import colorful as cf
from jinja2 import Template, Environment, FileSystemLoader

class GetFeedStaticData(object):
    def __init__(self, eachUrl):
        self.feeds = []
        self.dir_path = os.path.dirname(os.path.realpath(__file__))
        self.env = Environment(loader=FileSystemLoader('{}/templates'.format(self.dir_path)))
        self.fetch_feeds(eachUrl)
        self.empty_public()
        self.copy_static()
        self.render_page()

    def fetch_feeds(self, URLs):
        """ Request and parse all of the feeds, saving them in self.feeds """
        for url in URLs:
          try:
            print(f"Fetching {url}")
            self.feeds.append(feedparser.parse(url))
            fetch_content(url)
          except Exception as er:
            print("")

    def empty_public(self):
        """ Ensure the public directory is empty before generating. """
        try:
            shutil.rmtree('./output') 
            os.mkdir('./output')
        except:
            print("Error cleaning up old files.")

    def copy_static(self):
        """ Copy static assets to the public directory """
        try:
            shutil.copytree('{}/templates/static'.format(self.dir_path), 'output/static')
        except:
            print("Error copying static files.")

    def render_page(self):
        print("Rendering page to static file.")
          template = self.env.get_template('_layout.html')
          print(cfail("Error while reading template: {}".format(str(error))))
        with open('output/index.html', 'w+', encoding='utf8') as file:
            html = template.render(
                title = "Spiffy Feeds",
                feeds = self.feeds
            )
            file.write(html)



#!/usr/bin/python 
# -*- coding: utf-8 -*-
import argparse
import sys
import logging
from distutils.util import strtobool

__version_info__ = (0,2,23)
__version__ = '.'.join(map(str,__version_info__))

__base_path__ = 'feeds.txt'

parser = argparse.ArgumentParser(description='News Feed Syndication - A package that read and fetch RSS feeds from the publications.')
parser.add_argument("-V",
        "--version",
        action="store_true",
        help="Print the package version and quit"
        )
parser.add_argument('-v', '--verbose', help='Show verbose messages', action='count', default=0)
                        
parser.add_argument('-f','--filename',help="specify which file type to use (for example: nfsyndication-src --filename=./path/to/sample.file.txt)", nargs='+')
parser.add_argument('--outputJSON', help='Save feeds to output file JSON format.')
parser.add_argument("--comparator-filter", type=lambda x:bool(strtobool(x)),
       nargs='?', help='Enable the comparator. This will randomly ignore stale RSS feeds from the rendering output HTML.', const=True, default=False)


#!/usr/bin/env python
# coding=utf-8
import os
from os.path import abspath, realpath, split, dirname
import collections
from datetime import datetime, timedelta
import time

import feedparser
import jinja2
import json
import logging
import pytz
import sys
import logging
from . import parser

from jinja2.exceptions import UndefinedError
from . import parser
from .extras import fetch_content, templateContent, process_entry
from .styles import cssTextDecoded

args = parser.parse_args()

#argsFilename = args.filename

# Get a list of feed URLs
try:
 with open('feeds.txt') as f:
    SUBSCRIPTIONS = list(f)
    print('Loading feeds.txt')
except FileNotFoundError:   # If you don't have 'feeds.txt' in specified path, you can specify one (nfsyndication-src --filename=sample.txt)
 try:
  for documentList in args.filename:
   with open(documentList) as f:
    SUBSCRIPTIONS = list(f)
   print('Loading file: ' + documentList)
 except TypeError:
   raise Exception('NFSyndication [ERROR]: feeds.txt not found. See `nfsyndication-src --help` for more.')

posts = []
outJSONFeed= []

try:
 for url in SUBSCRIPTIONS:
    try:
        feed = feedparser.parse(url)
        blog = feed['feed']['title']
    except KeyError:
        if args.verbose and feed.bozo:
            logging.error("Feed data summary on URL {}".format(url))
            logging.error("Exception [{bozo_exception}]: {bozo_message}".format(bozo_exception=str(feed.bozo_exception.__class__.__name__), bozo_message=str(feed.bozo_exception)))
            if (hasattr(feed.bozo_exception, 'getLineNumber') and hasattr(feed.bozo_exception, 'getMessage')):
                line = feed.bozo_exception.getLineNumber()
                logging.error('Line %d: %s', line, feed.bozo_exception.getMessage())
                
        raise Exception(f"[{feed.bozo_exception}] (code {feed.status}) \n{(f'Could not fetch URL(s): {url}')}")
        sys.exit(-1)
        continue
    for entry in feed['entries']:
        post = process_entry(entry, blog)
        if post:
            posts.append(post)
            outJSONFeed.append(feed)
    try:
     fetch_content(url)
    except:
     raise SystemExit
except NameError:
  pass

if args.outputJSON:
      with open(args.outputJSON, 'w', encoding='utf8') as outf:
           json.dump(outJSONFeed, outf, ensure_ascii=False, indent=4)
  
# Get the template, and drop in the posts
dir_path = os.path.split(os.path.realpath(__file__))[0]

try:
 with open(f'{dir_path}/templates/template.html', encoding='utf8') as f:
    print("\nChecking original template file...")
    template = jinja2.Template(f.read())
 with open(f'output/index.html', 'w', encoding='utf8') as f:
      f.write(template.render(posts=posts, time=datetime.now()))
      print('Successful.')   
 with open("output/style.css", 'w') as f:
    f.write(cssTextDecoded)

except FileNotFoundError:
    template = jinja2.Template(templateContent)
# When done, it converts to HTML
    with open(f'output/index.html', 'w', encoding='utf8') as f:
      f.write(template.render(cssText=cssTextDecoded, posts=posts, time=datetime.now()))
      print('Successful.')



import os
import sys
import argparse
import logging
import time
import logging.handlers
from . import __version__, __base_path__
from . import config
from . import parser

from functools import wraps

handler = logging.StreamHandler()

verbose = False

args = parser.parse_args()

def exec_wrapper(func):
    """This decorator prints the execution time for the decorated function."""

    @wraps(func)
    def wrapper(*args, **kwargs):
        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        print("Command execution took {}s".format(round(end - start, 2)))
        return result

    return wrapper

args = parser.parse_args()
if args.version:
      print('NFSyndication version: ' + __version__)
      sys.exit(0)

@exec_wrapper
def run():
  try:
   #logger.setLevel(logging.DEBUG)
   if os.path.isfile(__base_path__) and os.access(__base_path__, os.R_OK):
    os.mkdir('output')
  except FileExistsError:
    # directory already exists
   pass
  return config.init()

#logging.StreamHandler()
#logger = logging.getLogger()

logger = logging.getLogger()
formatter = logging.Formatter('%(asctime)s | %(levelname)s | %(message)s')

stdout_handler = logging.StreamHandler(sys.stdout)
stdout_handler.setLevel(logging.DEBUG)
stdout_handler.setFormatter(formatter)

if args.verbose == 1:
        logger.setLevel(logging.INFO)
        file_handler = logging.FileHandler('logs.log')
        file_handler.setLevel(logging.DEBUG)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
        logger.addHandler(stdout_handler)
elif args.verbose == 2:
        logger.setLevel(logging.DEBUG)
elif args.verbose == 3:
        logger.setLevel(logging.CRITICAL)
        

if __name__ == '__main__':
    run()